import {
  computed,
  ref,
  defineComponent,
  onMounted,
  reactive,
  onUnmounted
} from 'vue'
import { Card, Cell, CellGroup, Popup, SubmitBar, Toast } from 'vant'
import { addressType, cartConfirm, formateAttr } from '../cart'
import styles from '../index.module.less'
import Address from '../components/address'
import request from '@/helpers/request'
import { useRouter } from 'vue-router'
import ColProtocol from '@/components/col-protocol'
import Payment from '@/views/order-detail/payment'
import { state } from '@/state'
import ColPopup from '@/components/col-popup'
import UserAuth from '@/views/order-detail/userAuth'
import ColResult from '@/components/col-result'
import { moneyFormat } from '@/helpers/utils'
import {
  listenerMessage,
  removeListenerMessage
} from '@/helpers/native-message'
export default defineComponent({
  name: 'cartConfirm',
  setup() {
    const list = cartConfirm.cartPromotionItemList
    const calcAmount = cartConfirm.calcAmount

    const address = ref<addressType>()
    if (cartConfirm.memberReceiveAddressList.length) {
      const a =
        cartConfirm.memberReceiveAddressList.find(
          (n: any) => n.defaultStatus
        ) || cartConfirm.memberReceiveAddressList[0]
      if (a) address.value = a
    }
    const setAddress = (result: addressType) => {
      address.value = result
    }
    onMounted(() => {
      listenerMessage('getAddress', result => {
        setAddress(result?.content || {})
      })
    })
    onUnmounted(() => {
      removeListenerMessage('getAddress', () => {})
    })

    const agreeStatus = ref(false)
    const paymentPopup = ref(false)
    const authPopup = ref(false)
    const orderInfo = reactive({
      orderNo: '',
      actualPrice: 0
    })
    // 提交
    const onSubmit = () => {
      if (!address.value?.id) {
        Toast('请选择收货地址')
        return
      }
      if (!agreeStatus.value) {
        Toast('请先阅读并同意《酷乐秀平台服务协议》')
        return
      }
      const users = state.user.data
      // 判断是否需要实名认证
      if (!users?.realName || !users?.idCardNo) {
        authPopup.value = true
        return
      }

      // 判断是否有订单号
      if (orderInfo.orderNo) {
        paymentPopup.value = true
        return
      }
      createOrder()
    }
    const router = useRouter()
    //创建订单
    const createOrder = async () => {
      const ids = list.reduce((arr, value: any) => {
        arr.push(value.id)
        return arr
      }, [])
      const body = {
        cartIds: ids,
        memberReceiveAddressId: address.value?.id
      }
      try {
        let { code, data } = await request.post(
          '/api-mall-portal/order/generateOrder',
          { data: body }
        )
        if (code === 200) {
          if (data.order.status === 1) {
            router.replace({
              path: `/shopTrade`,
              query: {
                orderNo: data?.order.orderSn,
                id: data?.order.id
              }
            })
            return
          }
          paymentPopup.value = true
          orderInfo.orderNo = data?.order.orderSn || ''
          orderInfo.actualPrice = data?.order.payAmount || 0
        }
      } catch (error) {}
    }

    //认证成功
    const onAuthSuccess = () => {
      authPopup.value = false
      onSubmit() // 实名成功后自动支付
    }
    return () => (
      <>
        {list.length ? (
          <div class={styles.cartConfirm}>
            <div class={styles.cartConfirmBox}>
              <Address item={address.value} setAddress={setAddress} />
            </div>
            <div
              style={{ marginTop: '20px' }}
              class={[styles.cartBox, styles.cartConfirmBox]}
            >
              <div class={styles.shopBox}>
                {list.map((item: any) => (
                  <div
                    class={[styles.cartItem]}
                    style={{ marginBottom: '10px' }}
                  >
                    <Card
                      price={moneyFormat(item.price)}
                      desc={formateAttr(item.productAttr)}
                      title={item.productName}
                      thumb={item.productPic}
                      num={item.quantity}
                    ></Card>
                  </div>
                ))}
              </div>
              <CellGroup border={false}>
                <Cell
                  border={false}
                  title="总额"
                  value={'¥ ' + moneyFormat(calcAmount.totalAmount)}
                ></Cell>
                {/* <Cell
                  border={false}
                  title="运费"
                  value={moneyFormat(calcAmount.freightAmount)}
                ></Cell>
                <Cell
                  border={false}
                  title="优惠"
                  value={'-¥ ' + moneyFormat(calcAmount.promotionAmount)}
                ></Cell> */}
                {/* <Cell border={false} title="优惠卷" value="暂无可用优惠卷"></Cell>
            <Cell border={false} title="乐乐币抵扣" value={"-¥" + calcAmount.promotionAmount}></Cell> */}
              </CellGroup>
            </div>

            <div class={styles.payProtocol}>
              <ColProtocol v-model={agreeStatus.value}></ColProtocol>
            </div>
            <SubmitBar
              buttonText={`结算(${list.length})`}
              buttonColor="var(--van-primary)"
              disabled={list.length === 0}
              onSubmit={() => onSubmit()}
            >
              <div class={styles.confirmBottom}>
                合计{' '}
                <span class={styles['price-des']}>
                  ¥{moneyFormat(calcAmount.payAmount)}
                </span>
              </div>
            </SubmitBar>
            <div style={{ height: 'var(--van-submit-bar-height)' }}></div>
            <ColPopup v-model={authPopup.value}>
              <UserAuth onSuccess={onAuthSuccess} />
            </ColPopup>
            <Popup
              show={paymentPopup.value}
              closeOnClickOverlay={false}
              position="bottom"
              round
              closeOnPopstate
              safeAreaInsetBottom
              style={{ minHeight: '30%' }}
            >
              <Payment
                v-model={paymentPopup.value}
                orderInfo={orderInfo}
                paymentType="goodsPay"
                onBackOut={() => (paymentPopup.value = false)}
              />
            </Popup>
          </div>
        ) : (
          <ColResult
            buttonText="去购物车"
            onClick={() => {
              router.push({ path: '/cart' })
            }}
          ></ColResult>
        )}
      </>
    )
  }
})
